==> 0.0.8.tar.gz: lib/nui_miniprogram_engine.dart <==
import 'dart:async';
import 'dart:convert';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';

//Concept
/*
1) Send a message
2) Wait for response
3)
 */

typedef NUICrossEventMessageListener = NUIMiniProgramMessage Function(NUIMiniProgramMessage message);
typedef NUIMiniProgramChannelListener = NUIMiniProgramMessage Function(String data, NUIMiniProgramMessage message);

class NUIMiniProgramChannel{
  NUIMiniProgramChannel(String channel, {NUIMiniProgramChannelListener listener}){
    this._channel = channel;
    this._listener = listener;
  }
  String _channel;
  NUIMiniProgramChannelListener _listener;

  String get channel => _channel;
}


abstract class NUIMiniProgramJSEngineInterface{
  Map<String, Completer<NUIMiniProgramMessage>> _messageQueue = Map();
  List<NUIMiniProgramChannel> _channels = [];
  List<NUIMiniProgramChannel> _internalChannels = [];

  void initInternalChannelsForWeb(BuildContext context){
    _internalChannels = [
      NUIMiniProgramChannel(NUIMiniProgramWebEventType.CAN_POP.value, listener: (data, message){
        print("Received message from ${NUIMiniProgramWebEventType.CAN_POP.value}, payload: $data");
        final canPop = Navigator.canPop(context);
        return NUIMiniProgramMessage(id: message.id, data: canPop ? "true": false, channel: NUIMiniProgramWebEventType.CAN_POP.value);
      }),
    ];
  }

  void setChannels(List<NUIMiniProgramChannel> channels){
    this._channels.clear();
    this._channels.addAll(channels);
  }

  List<NUIMiniProgramChannel> get channels => _channels;

  void messageReceived(String messageRaw) async{
    final dataMessage = processEventMessage(messageRaw);
    if(dataMessage != null) {
      completeMessageInQueue(dataMessage);
      final reply = await _redirectPlatformMessage(dataMessage);
      if(reply != null){
        print("NUIMiniProgramEngine: Replying message with : ${jsonEncode(reply)}");
        await _sendCrossEvent(reply);
      }
    }
  }

  Future<NUIMiniProgramMessage> _redirectPlatformMessage(NUIMiniProgramMessage message) async{
    //The message is a received response, so ignore this as an internal message
    if(message.isResponseReceived()){
      print("NUIMiniProgram: Just a response received message, so ignore and stop the conversation");
      return null;
    }

    _internalChannels?.forEach((element) {
      if(match(element.channel, message.channel)){
        print("NUIMiniProgramEngine: Found a match for channel ${message.channel} for internal");
        if (element._listener != null) {
          //User has a custom listener to this event, check if there is a custom response or use the default received response
          final result = element._listener(message.data, message);
          return result ?? message.responseReceived();
        }
        else{
          return message.responseReceived();
        }
      }
    });
    _channels?.forEach((element) {
      if(match(element.channel, message.channel)) {
        print("NUIMiniProgramEngine: Found a match for channel ${message.channel} for external");
        if (element._listener != null) {
          //User has a custom listener to this event, check if there is a custom response or use the default received response
          final result = element._listener(message.data, message);
          return result ?? message.responseReceived();
        }
        else{
          return message.responseReceived();
        }
      }
    });
    return message.responseReceived();
  }

  void addMessageToQueue(NUIMiniProgramMessage message){
    _messageQueue[message.id] = Completer();
  }

  void completeMessageInQueue(NUIMiniProgramMessage message){
    print("NUIMiniProgramEngine: Completing message of id ${message.id} from queue");
    final completed = _messageQueue[message.id];
    // if(completed != null && completed.isCompleted){
    //   _messageQueue[message.id] = Completer();
    // }
    _messageQueue[message.id]?.complete(message);
  }

  Completer<NUIMiniProgramMessage> _getResponseMessageQueue(String id){
    final messageQueue = _messageQueue[id];
    if(messageQueue == null) return null;
    return messageQueue;
  }

  Future<bool> sendPlatformCrossEvent(NUIMiniProgramMessage message);

  Future<bool> _sendCrossEvent(NUIMiniProgramMessage message) async{
    try {
      addMessageToQueue(message);
      await sendPlatformCrossEvent(message);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngineInterface] Failed to send cross event with error : $e");
      return false;
    }
  }

  Future<NUIMiniProgramMessage> sendAndAwaitCrossEvent(NUIMiniProgramMessage message) async{
    final sent = await _sendCrossEvent(message);
    if(sent) {
      final queue = _getResponseMessageQueue(message.id);
      if (queue == null) return null;
      return queue.future;
    }
    return null;
  }

  NUIMiniProgramMessage processEventMessage(String message){
    try{
      final decoded = jsonDecode(message);
      print("NUIMiniProgramCore: Processing event message: $message");
      final programMessage = NUIMiniProgramMessage().fromJson(decoded);
      if(programMessage.channel == null || programMessage.data == null){
        return null;
      }
      return programMessage;
    }catch(e){
      return null;
    }
  }
}

==> 0.0.9.tar.gz: lib/nui_miniprogram_engine.dart <==
import 'dart:async';
import 'dart:convert';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';

//Concept
/*
1) Send a message
2) Wait for response
3)
 */

typedef NUICrossEventMessageListener = NUIMiniProgramMessage Function(NUIMiniProgramMessage message);
typedef NUIMiniProgramChannelListener = NUIMiniProgramMessage Function(String data, NUIMiniProgramMessage message);

class NUIMiniProgramChannel{
  NUIMiniProgramChannel(String channel, {NUIMiniProgramChannelListener listener}){
    this._channel = channel;
    this._listener = listener;
  }
  String _channel;
  NUIMiniProgramChannelListener _listener;

  String get channel => _channel;
}


abstract class NUIMiniProgramJSEngineInterface{
  Map<String, Completer<NUIMiniProgramMessage>> _messageQueue = Map();
  List<NUIMiniProgramChannel> _channels = [];
  List<NUIMiniProgramChannel> _internalChannels = [];

  void initInternalChannelsForWeb(BuildContext context){
    _internalChannels = [
      NUIMiniProgramChannel(NUIMiniProgramWebEventType.CAN_POP.value, listener: (data, message){
        print("Received message from ${NUIMiniProgramWebEventType.CAN_POP.value}, payload: $data");
        final canPop = Navigator.canPop(context);
        print("Can pop for web : $canPop");
        final reply = NUIMiniProgramMessage(id: message.id, data: canPop ? "true": "false", channel: NUIMiniProgramWebEventType.CAN_POP.value);
        print("Replying for internal Can Pop with message: ${jsonEncode(reply)}");
        return reply;
      }),
    ];
  }

  void setChannels(List<NUIMiniProgramChannel> channels){
    this._channels.clear();
    this._channels.addAll(channels);
  }

  List<NUIMiniProgramChannel> get channels => _channels;

  void messageReceived(String messageRaw) async{
    final dataMessage = processEventMessage(messageRaw);
    if(dataMessage != null) {
      completeMessageInQueue(dataMessage);
      final reply = await _redirectPlatformMessage(dataMessage);
      if(reply != null){
        print("NUIMiniProgramEngine: Replying message with : ${jsonEncode(reply)}");
        await _sendCrossEvent(reply);
      }
    }
  }

  Future<NUIMiniProgramMessage> _redirectPlatformMessage(NUIMiniProgramMessage message) async{
    //The message is a received response, so ignore this as an internal message
    if(message.isResponseReceived()){
      print("NUIMiniProgram: Just a response received message, so ignore and stop the conversation");
      return null;
    }

    _internalChannels?.forEach((element) {
      if(match(element.channel, message.channel)){
        print("NUIMiniProgramEngine: Found a match for channel ${message.channel} for internal");
        if (element._listener != null) {
          //User has a custom listener to this event, check if there is a custom response or use the default received response
          final result = element._listener(message.data, message);
          print("NUIMiniProgramEngine: Listener is not null with a result of : ${jsonEncode(result)}");
          return result ?? message.responseReceived();
        }
        else{
          return message.responseReceived();
        }
      }
    });
    _channels?.forEach((element) {
      if(match(element.channel, message.channel)) {
        print("NUIMiniProgramEngine: Found a match for channel ${message.channel} for external");
        if (element._listener != null) {
          //User has a custom listener to this event, check if there is a custom response or use the default received response
          final result = element._listener(message.data, message);
          return result ?? message.responseReceived();
        }
        else{
          return message.responseReceived();
        }
      }
    });
    return message.responseReceived();
  }

  void addMessageToQueue(NUIMiniProgramMessage message){
    _messageQueue[message.id] = Completer();
  }

  void completeMessageInQueue(NUIMiniProgramMessage message){
    print("NUIMiniProgramEngine: Completing message of id ${message.id} from queue");
    final completed = _messageQueue[message.id];
    if(completed != null && completed.isCompleted){
      _messageQueue[message.id] = Completer();
    }
    _messageQueue[message.id]?.complete(message);
  }

  Completer<NUIMiniProgramMessage> _getResponseMessageQueue(String id){
    final messageQueue = _messageQueue[id];
    if(messageQueue == null) return null;
    return messageQueue;
  }

  Future<bool> sendPlatformCrossEvent(NUIMiniProgramMessage message);

  Future<bool> _sendCrossEvent(NUIMiniProgramMessage message) async{
    try {
      addMessageToQueue(message);
      await sendPlatformCrossEvent(message);
      return true;
    }catch(e){
      print("[NUIMiniProgramJSEngineInterface] Failed to send cross event with error : $e");
      return false;
    }
  }

  Future<NUIMiniProgramMessage> sendAndAwaitCrossEvent(NUIMiniProgramMessage message) async{
    final sent = await _sendCrossEvent(message);
    if(sent) {
      final queue = _getResponseMessageQueue(message.id);
      if (queue == null) return null;
      return queue.future;
    }
    return null;
  }

  NUIMiniProgramMessage processEventMessage(String message){
    try{
      final decoded = jsonDecode(message);
      print("NUIMiniProgramCore: Processing event message: $message");
      final programMessage = NUIMiniProgramMessage().fromJson(decoded);
      if(programMessage.channel == null || programMessage.data == null){
        return null;
      }
      return programMessage;
    }catch(e){
      return null;
    }
  }
}
